@@ -1,4 +1,4 @@
-use anyhow::Result;
+use anyhow::{Context, Result};
 use clap::{App, Arg};
 use hgrep::grep::BufReadExt;
 use hgrep::printer::{PrinterOptions, TextWrapMode};
@@ -97,10 +97,6 @@ fn cli<'a>() -> App<'a> {
                 .takes_value(true)
                 .value_name("NUM")
                 .about("Width (number of characters) of terminal window"),
-        ).arg(
-            Arg::new("no-wrap")
-                .long("no-wrap")
-                .about("DEPRECATED: The same as `--wrap never`. This flag will be removed at v0.2.0")
         ).arg(
             Arg::new("wrap")
                 .long("wrap")
@@ -174,7 +170,12 @@ fn cli<'a>() -> App<'a> {
                     .long("smart-case")
                     .about("Search case insensitively if the pattern is all lowercase. Search case sensitively otherwise"),
             )
-            .arg(Arg::new("hidden").long("hidden").about("Search hidden files and directories. By default, hidden files and directories are skipped"))
+            .arg(
+                Arg::new("hidden")
+                    .short('.')
+                    .long("hidden")
+                    .about("Search hidden files and directories. By default, hidden files and directories are skipped"),
+            )
             .arg(
                 Arg::new("glob")
                     .short('g')
@@ -244,13 +245,6 @@ fn cli<'a>() -> App<'a> {
                     .value_name("NUM")
                     .about("Limit the depth of directory traversal to NUM levels beyond the paths given"),
             )
-            .arg(
-                Arg::new("max-filesize")
-                    .long("max-filesize")
-                    .takes_value(true)
-                    .value_name("NUM")
-                    .about("Ignore files larger than NUM in size"),
-            )
             .arg(
                 Arg::new("line-regexp")
                     .short('x')
@@ -286,6 +280,29 @@ fn cli<'a>() -> App<'a> {
                     .long("type-list")
                     .about("Show all supported file types and their corresponding globs"),
             )
+            .arg(
+                Arg::new("max-filesize")
+                    .long("max-filesize")
+                    .takes_value(true)
+                    .value_name("NUM+SUFFIX?")
+                    .about("Ignore files larger than NUM in size. This does not apply to directories.The input format accepts suffixes of K, M or G which correspond to kilobytes, megabytes and gigabytes, respectively. If no suffix is provided the input is treated as bytes"),
+            )
+            .arg(
+                Arg::new("invert-match")
+                    .short('v')
+                    .long("invert-match")
+                    .about("Invert matching. Show lines that do not match the given patterns"),
+            )
+            .arg(
+                Arg::new("one-file-system")
+                    .long("one-file-system")
+                    .about("When enabled, the search will not cross file system boundaries relative to where it started from"),
+            )
+            .arg(
+                Arg::new("no-unicode")
+                    .long("no-unicode")
+                    .about("Disable unicode-aware regular expression matching"),
+            )
             .arg(
                 Arg::new("PATTERN")
                     .about("Pattern to search. Regular expression is available"),
@@ -322,6 +339,72 @@ fn generate_completion_script(shell: &str) {
     }
 }
 
+#[cfg(feature = "ripgrep")]
+fn build_ripgrep_config(
+    min_context: u64,
+    max_context: u64,
+    matches: &clap::ArgMatches,
+) -> Result<ripgrep::Config<'_>> {
+    let mut config = ripgrep::Config::default();
+    config
+        .min_context(min_context)
+        .max_context(max_context)
+        .no_ignore(matches.is_present("no-ignore"))
+        .hidden(matches.is_present("hidden"))
+        .case_insensitive(matches.is_present("ignore-case"))
+        .smart_case(matches.is_present("smart-case"))
+        .glob_case_insensitive(matches.is_present("glob-case-insensitive"))
+        .pcre2(matches.is_present("pcre2")) // must be before fixed_string
+        .fixed_strings(matches.is_present("fixed-strings"))
+        .word_regexp(matches.is_present("word-regexp"))
+        .follow_symlink(matches.is_present("follow-symlink"))
+        .multiline(matches.is_present("multiline"))
+        .crlf(matches.is_present("crlf"))
+        .multiline_dotall(matches.is_present("multiline-dotall"))
+        .mmap(matches.is_present("mmap"))
+        .line_regexp(matches.is_present("line-regexp"))
+        .invert_match(matches.is_present("invert-match"))
+        .one_file_system(matches.is_present("one-file-system"))
+        .no_unicode(matches.is_present("no-unicode"));
+
+    let globs = matches.values_of("glob");
+    if let Some(globs) = globs {
+        config.globs(globs);
+    }
+
+    if let Some(num) = matches.value_of("max-count") {
+        let num = num
+            .parse()
+            .context("could not parse --max-count option value as unsigned integer")?;
+        config.max_count(num);
+    }
+
+    if let Some(num) = matches.value_of("max-depth") {
+        let num = num
+            .parse()
+            .context("could not parse --max-depth option value as unsigned integer")?;
+        config.max_depth(num);
+    }
+
+    if let Some(size) = matches.value_of("max-filesize") {
+        config
+            .max_filesize(size)
+            .context("coult not parse --max-filesize option value as file size string")?;
+    }
+
+    let types = matches.values_of("type");
+    if let Some(types) = types {
+        config.types(types);
+    }
+
+    let types_not = matches.values_of("type-not");
+    if let Some(types_not) = types_not {
+        config.types_not(types_not);
+    }
+
+    Ok(config)
+}
+
 #[derive(Clone, Copy, PartialEq, Eq)]
 enum PrinterKind {
     #[cfg(feature = "bat-printer")]
@@ -331,8 +414,6 @@ enum PrinterKind {
 }
 
 fn app() -> Result<bool> {
-    use anyhow::Context;
-
     let matches = cli().get_matches();
     if let Some(shell) = matches.value_of("generate-completion-script") {
         generate_completion_script(shell);
@@ -349,10 +430,7 @@ fn app() -> Result<bool> {
         "syntect" => PrinterKind::Syntect,
         #[cfg(not(feature = "syntect-printer"))]
         "syntect" => anyhow::bail!("--printer syntect is not available because 'syntect-printer' feature was disabled at compilation"),
-        p => anyhow::bail!(
-            "Unknown printer '{}', at --printer option. It must be one of 'bat' or 'syntect'",
-            p
-        ),
+        p => anyhow::bail!("Unknown printer '{}', at --printer option. It must be one of 'bat' or 'syntect'", p),
     };
 
     let min_context = matches
@@ -420,9 +498,6 @@ fn app() -> Result<bool> {
             unreachable!(); // Option value was validated by clap
         }
     }
-    if matches.is_present("no-wrap") {
-        printer_opts.text_wrap = TextWrapMode::Never;
-    }
 
     if matches.is_present("first-only") {
         printer_opts.first_only = true;
@@ -472,68 +547,17 @@ fn app() -> Result<bool> {
         unreachable!();
     }
 
+    #[cfg(feature = "ripgrep")]
+    if matches.is_present("type-list") {
+        let config = build_ripgrep_config(min_context, max_context, &matches)?;
+        config.print_types(io::stdout().lock())?;
+        return Ok(true);
+    }
+
     #[cfg(feature = "ripgrep")]
     if let Some(pattern) = matches.value_of("PATTERN") {
         let paths = matches.values_of_os("PATH");
-        let mut config = ripgrep::Config::default();
-        config
-            .min_context(min_context)
-            .max_context(max_context)
-            .no_ignore(matches.is_present("no-ignore"))
-            .hidden(matches.is_present("hidden"))
-            .case_insensitive(matches.is_present("ignore-case"))
-            .smart_case(matches.is_present("smart-case"))
-            .glob_case_insensitive(matches.is_present("glob-case-insensitive"))
-            .pcre2(matches.is_present("pcre2")) // must be before fixed_string
-            .fixed_strings(matches.is_present("fixed-strings"))
-            .word_regexp(matches.is_present("word-regexp"))
-            .follow_symlink(matches.is_present("follow-symlink"))
-            .multiline(matches.is_present("multiline"))
-            .crlf(matches.is_present("crlf"))
-            .multiline_dotall(matches.is_present("multiline-dotall"))
-            .mmap(matches.is_present("mmap"))
-            .line_regexp(matches.is_present("line-regexp"));
-
-        if matches.is_present("type-list") {
-            config.print_types(io::stdout().lock())?;
-            return Ok(true);
-        }
-
-        let globs = matches.values_of("glob");
-        if let Some(globs) = globs {
-            config.globs(globs);
-        }
-
-        if let Some(num) = matches.value_of("max-count") {
-            let num = num
-                .parse()
-                .context("could not parse \"max-count\" option value as unsigned integer")?;
-            config.max_count(num);
-        }
-
-        if let Some(num) = matches.value_of("max-depth") {
-            let num = num
-                .parse()
-                .context("could not parse \"max-depth\" option value as unsigned integer")?;
-            config.max_depth(num);
-        }
-
-        if let Some(num) = matches.value_of("max-filesize") {
-            let num = num
-                .parse()
-                .context("could not parse \"max-filesize\" option value as unsigned integer")?;
-            config.max_filesize(num);
-        }
-
-        let types = matches.values_of("type");
-        if let Some(types) = types {
-            config.types(types);
-        }
-
-        let types_not = matches.values_of("type-not");
-        if let Some(types_not) = types_not {
-            config.types_not(types_not);
-        }
+        let config = build_ripgrep_config(min_context, max_context, &matches)?;
 
         #[cfg(feature = "syntect-printer")]
         if printer_kind == PrinterKind::Syntect {
