@@ -1,7 +1,7 @@
 use crate::chunk::Files;
 use crate::grep::GrepMatch;
 use crate::printer::Printer;
-use anyhow::Result;
+use anyhow::{Context, Result};
 use grep_matcher::{LineTerminator, Matcher};
 use grep_pcre2::{RegexMatcher as Pcre2Matcher, RegexMatcherBuilder as Pcre2MatcherBuilder};
 use grep_regex::{RegexMatcher, RegexMatcherBuilder};
@@ -44,6 +44,9 @@ pub struct Config<'main> {
     pcre2: bool,
     types: Vec<&'main str>,
     types_not: Vec<&'main str>,
+    invert_match: bool,
+    one_file_system: bool,
+    no_unicode: bool,
 }
 
 impl<'main> Config<'main> {
@@ -158,11 +161,6 @@ impl<'main> Config<'main> {
         self
     }
 
-    pub fn max_filesize(&mut self, num: u64) -> &mut Self {
-        self.max_filesize = Some(num);
-        self
-    }
-
     pub fn pcre2(&mut self, yes: bool) -> &mut Self {
         self.pcre2 = yes;
         self
@@ -178,6 +176,43 @@ impl<'main> Config<'main> {
         self
     }
 
+    pub fn max_filesize(&mut self, input: &str) -> Result<&mut Self> {
+        if input.is_empty() {
+            anyhow::bail!("Size string must not be empty");
+        }
+
+        let i = input.len() - 1;
+        let (input, mag) = match input.as_bytes()[i] {
+            b'k' | b'K' => (&input[..i], 1 << 10),
+            b'm' | b'M' => (&input[..i], 1 << 20),
+            b'g' | b'G' => (&input[..i], 1 << 30),
+            _ => (input, 1),
+        };
+
+        let u: u64 = input
+            .parse()
+            .context("could not parse file size value as unsigned integer")?;
+
+        self.max_filesize = Some(u * mag);
+
+        Ok(self)
+    }
+
+    pub fn invert_match(&mut self, yes: bool) -> &mut Self {
+        self.invert_match = yes;
+        self
+    }
+
+    pub fn one_file_system(&mut self, yes: bool) -> &mut Self {
+        self.one_file_system = yes;
+        self
+    }
+
+    pub fn no_unicode(&mut self, yes: bool) -> &mut Self {
+        self.no_unicode = yes;
+        self
+    }
+
     fn build_walker(&self, mut paths: impl Iterator<Item = &'main OsStr>) -> Result<Walk> {
         let target = paths.next().unwrap();
 
@@ -206,7 +241,8 @@ impl<'main> Config<'main> {
             .max_depth(self.max_depth)
             .max_filesize(self.max_filesize)
             .overrides(overrides)
-            .types(self.build_types()?);
+            .types(self.build_types()?)
+            .same_file_system(self.one_file_system);
 
         if !self.no_ignore {
             builder.add_custom_ignore_filename(".rgignore");
@@ -221,7 +257,8 @@ impl<'main> Config<'main> {
             .case_insensitive(self.case_insensitive)
             .case_smart(self.smart_case)
             .word(self.word_regexp)
-            .multi_line(true);
+            .multi_line(true)
+            .unicode(!self.no_unicode);
 
         if self.multiline {
             builder.dot_matches_new_line(self.multiline_dotall);
@@ -264,6 +301,10 @@ impl<'main> Config<'main> {
                 .max_jit_stack_size(Some(10 * (1 << 20)));
         }
 
+        if !self.no_unicode {
+            builder.utf(true).ucp(true);
+        }
+
         if self.multiline {
             builder.dotall(self.multiline_dotall);
         }
@@ -286,7 +327,8 @@ impl<'main> Config<'main> {
             .binary_detection(BinaryDetection::quit(0))
             .line_number(true)
             .multi_line(self.multiline)
-            .memory_map(mmap);
+            .memory_map(mmap)
+            .invert_match(self.invert_match);
         if self.crlf {
             builder.line_terminator(LineTerminator::crlf());
         }
@@ -305,19 +347,24 @@ impl<'main> Config<'main> {
         Ok(builder.build()?)
     }
 
-    pub fn print_types<W: io::Write>(&self, mut out: W) -> Result<()> {
-        let types = self.build_types()?;
-        for def in types.definitions() {
-            out.write_all(def.name().as_bytes())?;
-            out.write_all(b": ")?;
-            let mut globs = def.globs().iter();
-            out.write_all(globs.next().unwrap().as_bytes())?;
-            for glob in globs {
-                out.write_all(b", ")?;
-                out.write_all(glob.as_bytes())?;
+    pub fn print_types<W: io::Write>(&self, out: W) -> Result<()> {
+        fn print<W: io::Write>(mut out: W, types: &Types) -> io::Result<()> {
+            for def in types.definitions() {
+                write!(out, "\x1b[1m{}\x1b[0m: ", def.name())?;
+                let mut globs = def.globs().iter();
+                out.write_all(globs.next().unwrap().as_bytes())?;
+                for glob in globs {
+                    out.write_all(b", ")?;
+                    out.write_all(glob.as_bytes())?;
+                }
+                out.write_all(b"\n")?;
             }
-            out.write_all(b"\n")?;
+            Ok(())
         }
+
+        use crate::io::IgnoreBrokenPipe;
+        let types = self.build_types()?;
+        print(out, &types).ignore_broken_pipe()?;
         Ok(())
     }
 }
@@ -702,7 +749,7 @@ mod tests {
         config.print_types(&mut buf).unwrap();
         let output = String::from_utf8(buf).unwrap();
 
-        let re = Regex::new(r"^\w+: .+(, .+)*$").unwrap();
+        let re = Regex::new(r"^\x1b\[1m\w+\x1b\[0m: .+(, .+)*$").unwrap();
         for line in output.lines() {
             assert!(re.is_match(line), "{:?} did not match to {:?}", line, re);
         }
@@ -897,4 +944,57 @@ mod tests {
             [[(1, 2), (3, 4)], [(1, 2), (3, 4)]],
         );
     }
+
+    #[test]
+    fn test_parse_max_filesize() {
+        let tests = &[
+            ("123", Ok(123)),
+            ("123k", Ok(123 * 1024)),
+            ("123m", Ok(123 * 1024 * 1024)),
+            ("123g", Ok(123 * 1024 * 1024 * 1024)),
+            ("123K", Ok(123 * 1024)),
+            ("123M", Ok(123 * 1024 * 1024)),
+            ("123G", Ok(123 * 1024 * 1024 * 1024)),
+            ("", Err("Size string must not be empty")),
+            (
+                "abc",
+                Err("could not parse file size value as unsigned integer"),
+            ),
+            (
+                "123kk",
+                Err("could not parse file size value as unsigned integer"),
+            ),
+            (
+                "-123k",
+                Err("could not parse file size value as unsigned integer"),
+            ),
+        ];
+
+        for (input, want) in tests.iter().copied() {
+            let mut c = Config::default();
+            let err = c.max_filesize(input).err();
+            match want {
+                Ok(want) => assert_eq!(
+                    c.max_filesize,
+                    Some(want),
+                    "wanted {} for {:?}",
+                    want,
+                    input
+                ),
+                Err(want) => {
+                    let err = err.unwrap_or_else(|| {
+                        panic!("wanted error {:?} but no error for {:?}", want, input)
+                    });
+                    let msg = format!("{}", err);
+                    assert!(
+                        msg.contains(want),
+                        "wanted error {:?} but got {:?} for {:?}",
+                        want,
+                        msg,
+                        input
+                    );
+                }
+            }
+        }
+    }
 }
